#!/usr/bin/python
# Author:   @BlankGodd

from pyfiglet import Figlet
from search import Search_Genius
from interact import Interact
from save import Save
from web import Webpage
import time, os


class Tool:
    """For using BLyrics as a script instead of a module"""
    def __init__(self):
        """Constructor for Script class
        
        Methods:
            - start: to start the main script
            - get_articles: to retrieve articles from the genius web page
            - print_articles: for printing articles to screen
            - get_Searching: to initiate the searching module
            - get_chart: to retrieve a chart of the top 10 songs
            - space_cre: for creating gaps in tables(charts)
        """
        banner = Figlet(font='standard')
        print(banner.renderText("BLyrics"))

        # i have to make objects for each class
        # the script isn't working without ding it like this
        self.search_bot = Search_Genius() 
        self.interact_bot = Interact()
        self.save_bot = Save()
        self.web_bot = Webpage()
        # and add positional arguments

        print()
        print("Welcome To BLyrics")
        print()
        print('If you are new to BLyrics please check the help guide by typing help')
        print('Ignore to continue')
        print()
        comd = input(": ").lower()
        if comd == 'help':
            print('Welcome to BLyrics')
            print("To go back, enter 'back'(lower case)")
            print("To return to start page, enter 'menu'(lower case)")
            print("To exit, enter 'ctrl+Z'")
            print('Follow the prompt and you should be fine :)')
            time.sleep(3)
            for i in range(2):
                print()
        self.start()

    def start(self):
        """A starting point for calling all commands
        """
        print()
        print("To get latest articles from Genius home page, press 1")
        print("To get a chart of the top songs, press 2")
        print('To get information about songs and artists, get song lyrics, press #/3')
        print()

        command = input(': ')
        if command == '1':
            self.get_articles()
        elif command == '2':
            self.get_chart()
        elif command == '3' or command == '#':
            self.get_searching()
        elif command == 'back' or 'command' == 'menu':
            self.start()
        else:
            print("Invalid input")
            self.start()
        self.start()
        
    def get_articles(self):
        """To get news and articles
        
        Properties:
            - Gets article title and link
        """
        print()
        articles_titles = self.web_bot.check_articles()
        if not articles_titles:
            print('Service Timeout. Please Retry!')
            print()
            self.start()
            return
        # (headline_d, other_news_d)
        headline_d = articles_titles[0]
        for k,v in headline_d.items():
            self.headline = k
            self.headline_link = v
        other_news_d = articles_titles[1]
        self.other_news_title = []
        self.other_news_link = []
        for k,v in other_news_d.items():
            self.other_news_title.append(k)
            self.other_news_link.append(v)
        self.print_articles()

    def print_articles(self):
        """For printing article to screen

         Returns:
            - Title: Article title
            - Body: Body of article
        """
        print()
        print('Headline')
        print('1. {}'.format(self.headline))
        print()
        print('Other News')
        pos = 2
        for i in self.other_news_title:
            print('{}. {}'.format(pos, i))
            pos += 1
        print()
        print('Enter article number to read')
        numb = input(': ')
        if numb == 'back' or numb == 'menu':
            self.start()
            return
        print('Getting Article...')
        article = ''
        article_title = ''
        if numb == '1':
            url = self.headline_link
            article = self.web_bot.get_article(url)
            if not article:
                print('Could not retrieve article. Please Retry')
                self.get_articles()
                return
            article_title = self.headline
        else:
            ind = int(numb)-2
            url = self.other_news_link[ind]
            article = self.web_bot.get_article(url)
            if not article:
                print('Could not retrieve article. Please Retry')
                self.get_articles()
                return
            article_title = self.other_news_title[ind]

        print()
        print('Title: {}'.format(article_title))
        print()
        print('Article: \n{}'.format(article))
        print()
        print()
        print('Read other articles or exit to menu')
        move = input('Read more(r) or exit (e): ').lower()
        if move == 'r':
            self.print_articles()
            return
        self.start()

    def get_chart(self):
        """For getting a chart of the top songs
        
        Properties:
            - Prints a table of top 10 songs and artists
        """
        print()
        chart = self.web_bot.get_charts()
        if not chart:
            print('Could not get Chart. Please Retry!')
            self.start()
            return
        ranks = chart[0]
        titles = chart[1]
        artists = chart[2]
        print()
        print('             CHARTS')
        print()
        print('Rank     Title                                    Artist')
        space = 0
        for i in range(len(ranks)):
            if len(titles[i]) < 30:
                space = 35 - len(titles[i])
                print(ranks[i],'     ',titles[i],self.space_cre(space),artists[i])
            else:
                tit = ''
                for j in range(25):
                    tit += titles[i][j]
                tit += '...'
                space = 35 - len(tit)
                print(ranks[i],'     ',tit,self.space_cre(space),artists[i])
            space = 0
        print()
        print('Check lyrics or information for top songs and artists now')

    def space_cre(self, n):
        """For creating gaps in tables(charts)
        
        Params:
            - n: number of spaces to be created/returned
        """
        for i in range(n):
            return ' ' * n

    def get_searching(self):
        """To starting searching
        
        Properties:
            Artist:
                -Name: Artist name
                -Alias: Artist aliases
                -Description: Information about artist
                -Twitter: Twitter handle
                -Instagram: Instagram handle
                -Facebook: Facebook name
                -Songs: List of top songs
            Songs:
                -Title: Song title
                -Artist: Artist name
                -Description: Song description
                -Release Date: Song release date
                -Recording Location
                -Lyrics: Song lyrics
        """
        print()
        print('Information')
        print('1: Search Artist')
        print('2: Search Song ')
        print()
        command = input('Enter 1 or 2: ').lower()
        if command == 'back' or command == 'menu':
            self.start()
            return
        print()
        response = None
        search_str = ''
        if command == '1':
            while search_str == '':
                print("Please enter valid input!")
                search_str = input('Enter Artist name: ')
            if search_str == 'back' or search_str == 'menu':
                self.start()
                return
            response = self.search_bot.search_artist(search_str = search_str)
        elif command == '2':
            while search_str == '':
                print("Please enter valid input!")
                search_str = input('Enter Song title: ')
            if search_str == 'back' or search_str == 'menu':
                self.start()
                return
            response = self.search_bot.search_song(search_str = search_str)
        else:
            print('Invalid input!')
            self.get_searching()
            return

        if not response:
            print('{} not found. (Service Timeout) Please retry!'.format(search_str))
            self.get_searching()
            return

        if command == '1':
            name = response['artist_name']
            alias = response['Aliases']
            twitter = response['Twitter Handle']
            ig = response['Instagram Handle']
            facebook = response['Facebook Name']
            description = response['Description']
            songs = response['songs']
            
            print()
            print('Artist Name: {}'.format(name))
            print('Aliases: {}'.format(alias))
            print()
            print('Twitter Handle: {}'.format(twitter))
            print('Instagram Handle: {}'.format(ig))
            print('Facebook Name: {}'.format(facebook))
            print()
            print('Description: \n{}'.format(description))
            print()
            print('Songs by {}'.format(name))
            if type(songs) == list:
                for song in songs:
                    print(song)
            else:
                for k,v in songs.items():
                    print(v)   
        else:
            title = response['Title']
            artist = response['Artist']
            description = response['Description']
            lyrics = response['Lyrics']
            recording_location = response['recording_location']
            release_date = response['release_date']
            song_id = response['song_id']

            print()
            print('Song Title: {}'.format(title))
            print('Artist: {}'.format(artist))
            print()
            print('Record Location: {}'.format(recording_location))
            print('Release Date: {}'.format(release_date))
            print()
            print('Description: \n{}'.format(description))
            print()
            print('Lyrics: \n{}'.format(lyrics))
            print()
            print()

            print('Would you like to view annotations')
            an = input('y/n: ').lower()
            print()
            if an == 'back' or an == 'menu':
                self.start()
                return
            if an == 'y':
                while True:
                    referents = self.interact_bot.get_referents(song_id = song_id)
                    if not referents:
                        print('Could not get referents')
                        break
                    num = len(referents)
                    annotations = {}
                    for i in range(num):
                        annotator_name = referents[i][0]
                        fragment = referents[i][3]
                        annotation = referents[i][5]
                        ls = (annotator_name, fragment, annotation)
                        annotations[i+1] = ls
                    print()
                    print('There are {} annotations for the track'.format(num))
                    print('1: View all annotations')
                    print('2: Cancel')
                    commandd = input(': ')
                    if commandd == '1':
                        for k,v in annotations.items():
                            print()
                            print('---------------------------------------------------------------------------------')
                            print(k)
                            print('Annotator: {}'.format(v[0]))
                            print()
                            print('Highlight: \n{}'.format(v[1]))
                            print()
                            print('Annotations/Notes \n{}'.format(v[2]))
                            print('---------------------------------------------------------------------------------')
                    elif commandd == 'back' or commandd == 'menu':
                        self.start()
                        return
                    else:
                        pass
                    break
            else:
                pass

        print()
        print('Would you like to save information to file')
        sv = input('y/n: ').lower()
        if sv == 'y':
            if command == '1':
                self.save_bot.save_artist(tbs = response)
            else:
                self.save_bot.save_song(tbs = response)
            print('File saved to BLyrics_Files directory')
        self.start()
        
              




#!/usr/bin/python
# Author:   @BlankGodd

from search import Search_Genius
from interact import Interact
from save import Save
from web import Webpage
from tool import Tool


class ResponseError(Exception):
    """Response error class"""
    pass

class Package:
    """class for using blyrics as a module"""
    def __init__(self):
        """Constructor for package class
        
        Methods:
            - search_song: to search for songs
            - save_song: to save song infomation to file
            - search_artist: to search for artist
            - save artist: to save artist information to file
            - get_annotations: for getting song annotations
            - get_article_links: For getting links and titles of articles currently on the genius home page
            - get_article: For getting an articles currently on the genius home page
            - get_chart: For getting chart of top trending songs
        """
        self.search_bot = Search_Genius()
        self.save_bot = Save()
        self.interact_bot = Interact()
        self.web_bot = Webpage()
    
    def search_song(self, song):
        """For getting song info and lyrics
        
        Params:
            - song: song title
        Returns:
            - dict: a dictionary of song information and lyrics
        """
        response = self.search_bot.search_song(search_str=song)
        if not response:
            raise ResponseError('Could not retrieve response')
        return response

    def save_song(self, song_info):
        """For saving song info
        
        Params:
            - song_info: value returned by search_song
        """
        self.save_bot.save_song(tbs=song_info)

    def search_artist(self, artist):
        """For getting artist info 
        
        Params:
            - artist: artist name
        Returns:
            - dict: a dictionary of artist information
        """
        response = self.search_bot.search_artist(search_str=artist)
        if not response:
            raise ResponseError('Could not retrieve response')
        return response

    def save_artist(self, artist_info):
        """For saving artist info
        
        Params:
            - artist_info: value returned by search_artist
        """
        self.save_bot.save_artist(tbs=artist_info)

    def get_annotations(self, song_id):
        """For getting song annotations
        
        Params:
            - song_id: song id gotten from search_song
        Returns:
            - annotations: list of annotations and other important information
        """
        annots = self.interact_bot.get_referents(song_id=song_id)
        if not annots:
            raise ResponseError('Could not retrieve response')
        return annots

    def get_articles_links(self):
        """For getting links and titles of articles currently on the genius home page
        
        Returns:
            - tuple: headliner and other articles
        """
        articles = self.web_bot.check_articles()
        if not articles:
            raise ResponseError('Could not retrieve response')
        return articles

    def get_article(self, link):
        """For getting an articles currently on the genius home page
        
        Params:
            - link: article link gotten from get_article_links
        Returns:
            - str: article content
        """
        article = self.web_bot.get_article(aritcle_link = link)
        if not article:
            raise ResponseError('Could not retrieve response')
        return article

    def get_chart(self):
        """For getting chart of top trending songs
        
        Returns:
            - tuple: ranks, song title and artist name
        """
        chart = self.web_bot.get_charts()
        if not chart:
            raise ResponseError('Could not retrieve response')
        return chart


if __name__ == '__main__':
    Tool()





#!/usr/bin/python
# Author:   @BlankGodd

import json
import ast
import requests

class DupError(Exception):
    """Duplication Error"""
    pass

class Interact:
    """For getting referents"""
    def __init__(self):
        """Interact class constructor
        
        Method:
            - get_referents: for getting referents and annotations 
                Params:
                    - song_id
                    - creator_id
        """

        self.access_token = 'pX_ZcyoBxAKt8Z2F9oCOASPTzspv9er17wWCAPNIwIWcr5Lg_AyMRgGsx846LVAE'
        self.root = 'https://api.genius.com'

    def get_referents(self, song_id = None, webpage_id = None,
                        creator_id = None):
        """For getting referents and annotations to a song
        
        Params:
            - song_id = None
            - creator_id = None
            ('Must provide song or creator id BUT not both)

        Returns:
            - list: list containing tuples of referents, annotations and other information
        """
        path = 'referents?'
        request_url = '/'.join([self.root, path])

        if (song_id and creator_id) or (song_id and webpage_id) or (webpage_id and song_id):
            raise DupError('Provide only one id')

        print('Getting Referents...')
        params = {'song_id' : song_id, 'web_page_id' : webpage_id,
                    'created_by_id' : creator_id, 'text_format' : 'plain',
                    'per_page' : 23, 'page' : 1}
        access_token = 'Bearer {}'.format(self.access_token)
        headers = {'Authorization': access_token, 'application' : 'BLyrics',
            'User-Agent':'https://github.com/BlankGodd/BLyrics'}
        
        response = None
        i = 0
        while i < 3:    # try reconnecting 2 times if status_Code != 200
            try:
                response = requests.get(request_url, params=params, headers=headers)
                if response.status_code == 200:
                    print('Request Successful...')
                    print()
                    break
            except:
                pass
            i += 1
        if response == None:
            return response

        referents = response.text
        referents = json.loads(response.text)
        
        referents_total = []
        num = len(referents['response']['referents'])
        for i in range(num):
            annotator_name = referents['response']['referents'][i]['annotator_login']
            annotator_id = referents['response']['referents'][i]['annotator_id']
            song_id_from = referents['response']['referents'][i]['annotatable']['id']
            fragment = referents['response']['referents'][i]['fragment']
            annotations_id = referents['response']['referents'][i]['annotations'][0]['id']
            annotation = referents['response']['referents'][i]['annotations'][0]['body']['plain']
            ls = (annotator_name, annotator_id, song_id_from, fragment,
                    annotations_id, annotation)
            referents_total.append(ls)

        return referents_total


    




#!/usr/bin/python
# Author:   @BlankGodd

import requests
from bs4 import BeautifulSoup
import re, ast

class Webpage:
    """For getting articles from www.genius.com"""

    def __init__(self):
        """Constructor for Webpage class
        
        Methods:
            - get_page: get the genius home page
            - check_articles: get top 5 articles(names and links)
            - get_article: get the complete article based on the link
            - get_charts: get top 10 songs and artist name
        """

        self.access_token = 'pX_ZcyoBxAKt8Z2F9oCOASPTzspv9er17wWCAPNIwIWcr5Lg_AyMRgGsx846LVAE'
        self.url = 'https://genius.com'

    def get_page(self, url):
        """For retriving a page

        Params:
            - url: url of page to be retrieved
        Returns:
            - response: response from request.get(url)
        """
        headers = {'User-Agent': 'https://github.com/BlankGodd/Blyrics',
                    'application': 'BLyrics'}
        i = 0
        while i < 3: # try reconnecting 2 times if status_Code != 200
            try:
                response = requests.get(url, headers=headers)
                if response.status_code == 200:
                    return response
            except:
                pass
            i += 1
        print()
        print("Could not retrieve page. Please check connection and retry!")
        print()
        return None

    def check_articles(self):
        """For scrapping article title and links
        
        Returns
            - tuple: contains dictionary with title as keys and links as values
        """
        print('Getting Articles...')
        response = self.get_page(self.url)
        if not response:
            return None
        print()
        html = BeautifulSoup(response.text, 'html.parser')
        headline = html.find('div', class_='EditorialPlacement__Title-sc-11ot04a-1 elKqNh').get_text()
        link =  html.find('a', class_= 'EditorialPlacement__Link-sc-11ot04a-2 bCpoMC')
        headline_link = link['href']
        
        other_news_raw = html.find_all('div', class_='EditorialPlacement__Title-sc-11ot04a-1 ABTJt')
        other_links_raw = html.find_all('a', class_="EditorialPlacement__Link-sc-11ot04a-2 hyRooK")
        other_news = []
        other_links = []
        for val in other_news_raw:
            other_news.append(val.text)
        for val in other_links_raw:
            other_links.append(val['href'])
        print()
        print('Request Successful...')
        print()

        headline_d = {headline:headline_link}
        other_news_d = {}
        for i in range(len(other_links)):
            other_news_d[other_news[i]] = other_links[i]

        articles = (headline_d, other_news_d)

        return articles
    
    def get_article(self, article_link):
        """For getting articles content
        
        Params:
            - article_link: link to article page
        Returns:
            - str: article content
        """
        article = self.get_page(article_link)
        if not article:
            return None
        html = BeautifulSoup(article.text, 'html.parser')
        article = html.find('div', class_="article_rich_text_formatting").get_text()
        article = re.sub(r'[\(\[].*?[\)\]]', '', article)
        return article
       
    def get_charts(self):
        """For getting chart of trending songs

        Returns:
            - tuple: ranks, song title and artist name
        """
        print()
        print('Getting Chart...')
        page = self.get_page(self.url)
        if not page:
            return None
        html = BeautifulSoup(page.text, 'html.parser')
        ranks_raw = html.find_all('div', class_="ChartItemdesktop__Rank-sc-3bmioe-1 tDViA")
        titles_raw = html.find_all('div', class_='ChartSongdesktop__Title-sc-18658hh-3 fODYHn')
        artists_raw = html.find_all('h4', class_='ChartSongdesktop__Artist-sc-18658hh-5 kiggdb')
        ranks = []
        song_titles = []
        artists = []
        for val in ranks_raw:
            ranks.append(val.text)
        for val in titles_raw:
            song_titles.append(val.text)
        for val in artists_raw:
            artists.append(val.text)
        print()
        print('Request Successful...')
        
        return ranks, song_titles, artists






    


#!/usr/bin/python
# Author:   @BlankGodd

from setuptools import setup

with open('description.rst', 'r') as fh:
    long_description = fh.read()

setup(
    name="blyrics",
    version="1.1",
    author="Damilare Agbabiaka (BlankGodd)",
    author_email="blankgodd33@gmail.com",
    description="Python client for the GENIUS API -     Also for getting latest articles and charts",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/BlankGodd/BLyrics",
    py_modules=["blyrics","interact","save","search","tool","web"],
    package_dir = {"":"blyrics"},
    install_requires=[
        "requests~=2.21.0",
        "pyfiglet~=0.8.post1",
        "beautifulsoup4~=4.9.0",
    ],
    classifiers=[
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Utilities",
        "Topic :: Artistic Software",
        "Topic :: Games/Entertainment",
        "Topic :: Internet",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content :: Content Management System",
        "Topic :: Internet :: WWW/HTTP :: Indexing/Search",
        "Topic :: Multimedia :: Sound/Audio"
    ],  
    python_requires='>=3.6',
    project_urls={
        "Bug Reports": "https://github.com/BlankGodd/BLyrics/issues",
        "Read the Docs": "https://github.com/BlankGodd/BLyrics/wiki",
    },
    keywords=["music", "lyrics", "mp3", "music charts", "entertainment",
             "genius", "articles"],
)


#!/usr/bin/python
# Author:   @BlankGodd

import requests
import ast, json
from bs4 import BeautifulSoup
import re


class Search_Genius:
    """Songs, Artists and Search"""
    def __init__(self):
        """Constructor for search genius 

        Methods
            - search: to make a general search
            - search_song: searching for songs using song_id
                leverages the search method first
            - search_artist: searches for artist info using artist_id
                leverages the search method first
            - search_artist_song: searches for songs by artist
        """

        self.access_token = 'pX_ZcyoBxAKt8Z2F9oCOASPTzspv9er17wWCAPNIwIWcr5Lg_AyMRgGsx846LVAE'
        self.root = 'https://api.genius.com'

    def search(self, search_str):
        """To make a general query with the api
        
        Params:
            - search_str: value to be searched
        Returns:
            - response object
        """
        path = 'search/'
        request_url = '/'.join([self.root,path])
        print()

        params = {'q': search_str}
        access_token = 'Bearer {}'.format(self.access_token)
        headers = {'Authorization': access_token, 'application' : 'BLyrics',
            'User-Agent':'https://github.com/BlankGodd/BLyrics'}

        i = 0
        while i < 3: # try reconnecting 2 times if status_code != 200
            try:
                response = requests.get(request_url, params=params, headers=headers)
                if response.status_code == 200:
                    return response.text
            except:
                pass
            i += 1
        return None

    def search_song(self, search_str):
        """Search songs

        Params:
            - song title
        Return:
            - dict object
            - information about song
            - song lyrics
        """
        song = self.search(search_str)
        if not song:
            return None
        song = json.loads(song)
        
        # number of songs
        num = len(song['response']['hits'])
        songs = []
        # title, song_id, artist, artist_id, lyrics url,
        for i in range(num):
            title = song['response']['hits'][i]['result']['title_with_featured']
            song_id = song['response']['hits'][i]['result']['id']
            artist = song['response']['hits'][i]['result']['primary_artist']['name']
            artist_id = song['response']['hits'][i]['result']['primary_artist']['id']
            lyrics_url = song['response']['hits'][i]['result']['url']
            ls = (title, song_id, artist, artist_id, lyrics_url)
            songs.append(ls)
        print()
        for k in range(len(songs)):
            print(k+1,':', songs[k][0], 'by', songs[k][2])

        print()
        print('If preferred song not in list, search artist name')
        try:
            rank = int(input('Pick the number for intended song:\n'))
        except:
            return None
        print()
        ranked_song = songs[rank-1]
        print(ranked_song[0], 'by', ranked_song[2])
        print()
        print('Getting info...')
        
        # say we have a song id for no role models
        song_id = ranked_song[1]
        path = 'songs/{}'.format(song_id)
        request_url = '/'.join([self.root,path])

        params = {'text_format':'plain'}
        access_token = 'Bearer {}'.format(self.access_token)
        headers = {'Authorization': access_token, 'application' : 'BLyrics',
            'User-Agent':'https://github.com/BlankGodd/BLyrics'}
        i = 0
        information = None
        while i < 3:
            try:
                response = requests.get(request_url, params=params, headers=headers)
                if response.status_code == 200:
                    print('Request successful...')
                    information = response.text
                    information = json.loads(information)
                    break
            except:
                pass
            i += 1
        
        # after getting this we extract the info we need
        print()
        print('Getting Lyrics...')
        url = ranked_song[4]
        headers_ = {'application' : 'BLyrics',
            'User-Agent':'https://github.com/BlankGodd/BLyrics'}
        lyrics = None
        page_title = ''
        i = 0
        while i < 5:
            j = 0
            while j < 3:
                try:
                    response2 = requests.get(url, headers=headers_)
                    if response2.status_code == 200:
                        print('Request{} successful...'.format(i + 1))
                        print()
                        # i need to understand this part     
                        html = BeautifulSoup(response2.text, 'html.parser')
                        page_title = html.find('title').get_text()
                        lyrics = html.find('div', class_='lyrics').get_text()
                        lyrics = re.sub(r'[\(\[].*?[\)\]]', '', lyrics)
                        ##################################3
                        break
                except:
                    pass
                j += 1
            i += 1

        try:
            description = information['response']['song']['description']['plain']
            release_date = information['response']['song']['release_date_for_display']
            recording_location = information['response']['song']['recording_location']
        except:
            description = None
            recording_location = None
            release_date_for_display = None
        final = {'Title' : ranked_song[0], 'Artist' : ranked_song[2], 
            'Description' : description, 'Lyrics' : lyrics, 
            'song_id' : song_id, 'lyrics_url' : ranked_song[4],
            'page_title' : page_title, 'recording_location' : recording_location,
            'release_date' : release_date}

        return final

    def search_artist(self, search_str):
        """Search about a particular artist
        For getting information

        Params:
            - Artist name (str)
        Return:
            - dict object
            - Information about artist
        """
        artist_ = self.search(search_str)
        if not artist_:
            return None
        artist = json.loads(artist_)
        
        num = len(artist['response']['hits'])
        all_artist = []
        for i in range(num):
            name = artist['response']['hits'][i]['result']['primary_artist']['name']
            artist_id = artist['response']['hits'][i]['result']['primary_artist']['id']
            ls = (name, artist_id)
            all_artist.append(ls)
        for i in range(len(all_artist)):
            print(i+1, all_artist[i][0])

        print()
        print("If artist not on list, enter 'e-x'")
        try:
            rank = int(input('Pick the number of intended artist: \n'))
        except:
            return
        ranked_artist = all_artist[rank-1]
        print()
        artist_id = ranked_artist[1]
        print()
        print('Getting Info...')
        path = 'artists/{}'.format(artist_id)
        request_url = '/'.join([self.root, path])

        params = {'text_format':'plain'}
        access_token = 'Bearer {}'.format(self.access_token)
        headers = {'Authorization': access_token, 'application' : 'BLyrics',
            'User-Agent':'https://github.com/BlankGodd/BLyrics'}
        i = 0
        response = None
        while i < 3:
            try:
                response = requests.get(request_url, params=params, headers=headers)
                if response.status_code == 200:
                    print('Request successful...')
                    print()
                    break
            except:
                pass
            i += 1
        if response == None:
            return response
        information = response.text
        information = json.loads(information)

        aka = information['response']['artist']['alternate_names']
        twitter_name = information['response']['artist']['twitter_name']
        facebook_name = information['response']['artist']['facebook_name']
        instagram_name = information['response']['artist']['instagram_name']
        description = information['response']['artist']['description']['plain']
        image_url = information['response']['artist']['image_url']
        akas = ', '.join(aka)

        # dictionary id : song title
        artist_songs = self.search_artist_song(artist_id)

        if not artist_songs:
            num = len(artist['response']['hits'])
            artist_songs = []
            for i in range(num):
                song_title = artist['response']['hits'][i]['result']['title_with_featured']
                artist_songs.append(song_title)

        final = {'artist_name' : ranked_artist[0], 'Aliases' : akas, 
        'Twitter Handle' : twitter_name, 'Instagram Handle' : instagram_name, 
        'Facebook Name' : facebook_name, 'Description' : description,
        'songs' : artist_songs, 'image_url' : image_url}

        return final

    def search_artist_song(self, artist_id):
        """Searching for all songs by artist

        Params:
            - artist_id : Genius id for artist

        Returns:
            - dict object containing songs
            - key = song_id
            - value = song_title
        """
        path = 'artists/{}/songs'.format(artist_id)
        request_url = '/'.join([self.root, path])
        print('Getting songs by Artist...')

        params = {'sort':'popularity','per_page': 23, 'page': 1}
        access_token = 'Bearer {}'.format(self.access_token)
        headers = {'Authorization': access_token, 'application' : 'BLyrics',
            'User-Agent':'https://github.com/BlankGodd/BLyrics'}
        i = 0
        response = None
        while i < 3:
            try:
                response = requests.get(request_url, params=params, headers=headers)
                if response.status_code == 200:
                    print('Request successful...')
                    print()
                    break
            except:
                pass
            i += 1
        if response == None:
            return response
        returned = json.loads(response.text)

        final = {}
        num = len(returned['response']['songs'])
        for i in range(num):
            song_id = returned['response']['songs'][i]['id']
            song_title = returned['response']['songs'][i]['title_with_featured']
            final[song_id] = song_title
            
        return final
        

    




#!/usr/bin/python
# Author:   @BlankGodd

import os
import json

class Save:
    """For saving information to files"""

    def __init__(self):
        """Constructor for saving files

        Methods:
            - save_artist: saving artist information
            - save_song: saving song information
        """
        self.dir_name = 'Blyrics_Files'
        working_dir = os.getcwd()
        os.chdir('..')
        self.path = os.getcwd()
        os.chdir(working_dir)

    def save_artist(self, tbs):
        """For saving information about artist to a file
        
        Properties:
            -Name: Artist name
            -Alias: Artist aliases
            -Description: Information about artist
            -Twitter: Twitter handle
            -Instagram: Instagram handle
            -Image: Image Url
            -Songs: List of top songs
        Params:
            - tbs: information to be saved returned by search.search_artist()
        Returns:
            - str: good or bad save (not saved)
        """
        dir_path = self.path
        full_path = os.path.join(dir_path, self.dir_name)
        if os.path.exists(full_path):
            pass
        else:
            os.mkdir(full_path)

        songs = tbs['songs']
        if type(songs) == list:
            songs = '\n'.join(songs)
        else:
            song_list = []
            for k,v in songs.items():
                song_list.append(v)
            songs = '\n'.join(song_list)

        vars = {'Name': tbs['artist_name'],'Alias': tbs['Aliases'],
                'Description': tbs['Description'],'Twitter': tbs['Twitter Handle'],
                'Instagram': tbs['Instagram Handle'],'Image Url': tbs['image_url'],
                'Songs': songs}

        vars_txt = """
                \nName: {0}\nAlias: {1}
                \n\nDescription: {2}\n\nTwitter: {3}
                \nInstagram: {4}\nImage Url: {5}
                \n\nSongs: {6}
        """.format(tbs['artist_name'],tbs['Aliases'],tbs['Description'],
                    tbs['Twitter Handle'],tbs['Instagram Handle'],
                    tbs['image_url'],songs)
        
        print('Save to json(1) or txt(2)')
        typee = input('1/2: ')
        if typee == '1':
            f_name = '{}.json'.format(tbs['artist_name'])
            file_name = os.path.join(full_path, f_name)
            with open(file_name, 'w') as fn:
                json.dump(vars, fn)
        else:
            f_name = '{}.txt'.format(tbs['artist_name'])
            file_name = os.path.join(full_path, f_name)
            with open(file_name, 'w') as fn:
                fn.write(vars_txt)

        print()
        print('Saving complete...')

    def save_song(self, tbs):
        """For saving information about song and song lyrics
        
        Properties:
            -Title: Song title
            -Artist: Artist name
            -Description: Song description
            -Release Date: Song release date
            -Recording Location
            -Lyrics: Song lyrics
        Params:
            - tbs: information to be saved returned by search.search_song()
        Returns:
            - str: good or bad save (not saved)
        """
        dir_path = self.path
        full_path = os.path.join(dir_path, self.dir_name)
        if os.path.exists(full_path):
            pass
        else:
            os.mkdir(full_path)

        vars = {'Title': tbs['Title'],'Artist': tbs['Artist'],'Release Date': tbs['release_date'],
                'Recording Location': tbs['recording_location'],'Description': tbs['Description'],
                'Lyrics': tbs['Lyrics']}

        vars_txt = """
            \nTitle: {0}\nArtist: {1}\n\nRelease Date: {2}\nRecording Location: {3}
            \n\nDescription: {4}\n\nLyrics: {5}
        """.format(tbs['Title'],tbs['Artist'],tbs['release_date'],tbs['recording_location'],
                    tbs['Description'],tbs['Lyrics'])

        print('Save to json(1) or txt(2)')
        typee = input('1/2: ')
        if typee == '1':
            f_name = '{}.json'.format(tbs['Title'])
            file_name = os.path.join(full_path, f_name)
            with open(file_name, 'w') as fn:
                json.dump(vars, fn)
        else:
            f_name = '{}.txt'.format(tbs['Title'])
            file_name = os.path.join(full_path, f_name)
            with open(file_name, 'w') as fn:
                fn.write(vars_txt)

        print()
        print('Saving complete...')


